@@ -112,6 +112,7 @@ use std::io::{self, ErrorKind, BufWriter, Write};
 use std::net::{SocketAddr, ToSocketAddrs, Shutdown};
 use std::sync::Arc;
 use std::time::Duration;
+use cogo::coroutine::yield_now;
 pub use self::request::Request;
 pub use self::response::Response;
 pub use crate::net::{Fresh, Streaming};
@@ -240,7 +241,30 @@ impl<L: NetworkListener + Send + 'static> Server<L> {
                 let mut stream = t_c!(stream);
                 let w = worker.clone();
                 runtime::spawn_stack_size(move || {
-                    w.handle_connection(&mut stream)
+                    #[cfg(unix)]
+                        {
+                            stream.set_nonblocking(true);
+                            stream.reset_io();
+                            let time_out = w.handle_connection(&mut stream);
+                            match time_out {
+                                Some(v) => {
+                                    let d = std::time::Instant::now();
+                                    loop {
+                                        stream.reset_io();
+                                        w.handle_connection(&mut stream);
+                                        stream.wait_io();
+                                        if d.elapsed() > v {
+                                            return;
+                                        }
+                                    }
+                                }
+                                None => {
+                                    return;
+                                }
+                            }
+                        }
+                    #[cfg(not(unix))]
+                        w.handle_connection(&mut stream);
                 }, stack_size);
             }
         }, stack_size);
@@ -265,9 +289,13 @@ fn handle_task<H, L>(mut server: Server<L>, handler: H, tasks: usize) -> crate::
     debug!("tasks = {:?}", tasks);
     let pool = ListenerPool::new(server.listener);
     let worker = Worker::new(handler, server.timeouts);
-    let work = move |mut stream| worker.handle_connection(&mut stream);
+    let work = move |mut stream| {
+        worker.handle_connection(&mut stream);
+    };
 
-    let guard = runtime::spawn(move || pool.accept(work, tasks));
+    let guard = runtime::spawn(move || {
+        pool.accept(work, tasks);
+    });
 
     Ok(Listening {
         _guard: Some(guard),
@@ -288,7 +316,7 @@ impl<H: Handler + 'static> Worker<H> {
         }
     }
 
-    pub fn handle_connection<S>(&self, stream: &mut S) where S: NetworkStream + Clone {
+    pub fn handle_connection<S>(&self, stream: &mut S) -> Option<Duration> where S: NetworkStream + Clone {
         debug!("Incoming stream");
         self.handler.on_connection_start();
 
@@ -296,7 +324,7 @@ impl<H: Handler + 'static> Worker<H> {
             Ok(addr) => addr,
             Err(e) => {
                 info!("Peer Name error: {:?}", e);
-                return;
+                return None;
             }
         };
         //safety will forget copy s
@@ -305,15 +333,24 @@ impl<H: Handler + 'static> Worker<H> {
         let mut rdr = BufReader::new(stream2);
         let mut wrt = BufWriter::new(stream);
 
+        let mut timeout = None;
         while self.keep_alive_loop(&mut rdr, &mut wrt, addr) {
             if let Err(e) = self.set_read_timeout(*rdr.get_ref(), self.timeouts.keep_alive) {
                 info!("set_read_timeout keep_alive {:?}", e);
                 break;
             }
+            match self.timeouts.keep_alive.as_ref() {
+                None => {}
+                Some(v) => {
+                    timeout = Some(v.clone());
+                }
+            }
         }
         self.handler.on_connection_end();
         debug!("keep_alive loop ending for {}", addr);
+
         std::mem::forget(s);
+        timeout
     }
 
     fn set_read_timeout(&self, s: &dyn NetworkStream, timeout: Option<Duration>) -> io::Result<()> {
