@@ -85,7 +85,8 @@ impl<'a, W: Any> Response<'a, W> {
         r#try!(write!(&mut self.body, "{} {}\r\n", self.version, self.status));
 
         if !self.headers.has::<header::Date>() {
-            self.headers.set(header::Date(header::HttpDate(now_utc())));
+            let date = httpdate::HttpDate::from(std::time::SystemTime::now()).to_string();
+            self.headers.set_raw("Date",vec![date.into_bytes()]);
         }
 
         let body_type = match self.status {
@@ -128,7 +129,7 @@ impl<'a> Response<'a, Fresh> {
     /// Creates a new Response that can be used to write to a network stream.
     #[inline]
     pub fn new(stream: &'a mut (dyn Write + 'a), headers: &'a mut header::Headers) ->
-            Response<'a, Fresh> {
+    Response<'a, Fresh> {
         Response {
             status: status::StatusCode::Ok,
             version: version::HttpVersion::Http11,
